/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2014
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev$
 * $Date$
 * $Author$
 * 
 */

package de.unihalle.informatik.MiToBo.core.gui;

import ij.IJ;
import ij.ImagePlus;
import de.unihalle.informatik.Alida.operator.*;
import de.unihalle.informatik.Alida.admin.annotations.ALDMetaInfo;
import de.unihalle.informatik.Alida.admin.annotations.ALDMetaInfo.ExportPolicy;
import de.unihalle.informatik.Alida.annotations.Parameter;
import de.unihalle.informatik.Alida.dataio.provider.swing.components.ALDOperatorParameterPanel;
import de.unihalle.informatik.Alida.dataio.provider.swing.events.ALDSwingValueChangeListener;
import de.unihalle.informatik.MiToBo.core.datatypes.images.MTBImage;

/**
 * Implementation of panels for MiToBo operator parameters.
 * <p>
 * Contrary to the super class {@link ALDOperatorParameterPanel} here
 * images are treated in a special manner, e.g. open images are 
 * pre-selected as default values in configuration windows.	
 * 
 * @author Birgit Moeller
 */
@ALDMetaInfo(export=ExportPolicy.ALLOWED)
public class MTBOperatorParameterPanel extends ALDOperatorParameterPanel {

	/**
	 * Default constructor.
	 */
	public MTBOperatorParameterPanel(ALDOperator op, Parameter.ExpertMode mode,
			boolean topLevel, ALDSwingValueChangeListener listener) {
		super(op,mode,topLevel,listener);
	}
	
  @Override
  protected Object getDefaultValue( Boolean useRequired, 
  		Boolean useSupplemental, ALDOpParameterDescriptor descr) {
  	// check if parameter refers to required image
  	if (   (useRequired == null || useRequired.booleanValue()) 
  			&&  isImg( descr.getMyclass())) {
    	int [] winIDs = ij.WindowManager.getIDList();
  		if (winIDs == null || winIDs.length == 0)
  			return null;
  		return MTBImage.createMTBImage( IJ.getImage());
  	} 
  	return super.getDefaultValue(useRequired, useSupplemental, descr);
  }

  /**
   * Checks if image is of correct type.
   * 
   * @param cl    Class of object in question.
   * @return      True if class corresponds to valid image class.
   */
  private boolean isImg( Class<?> cl) {
  	return    MTBImage.class.isAssignableFrom( cl) 
  			   || ImagePlus.class.isAssignableFrom( cl);
  }
}
